/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

"use strict";

var EXPORTED_SYMBOLS = ["decodeEntities"];

var entities = new Map([
  ["AElig","Æ"],
  ["Aacute","Á"],
  ["Acirc","Â"],
  ["Agrave","À"],
  ["Alpha","Α"],
  ["Aring","Å"],
  ["Atilde","Ã"],
  ["Auml","Ä"],
  ["Beta","Β"],
  ["Ccedil","Ç"],
  ["Chi","Χ"],
  ["Dagger","‡"],
  ["Delta","Δ"],
  ["ETH","Ð"],
  ["Eacute","É"],
  ["Ecirc","Ê"],
  ["Egrave","È"],
  ["Epsilon","Ε"],
  ["Eta","Η"],
  ["Euml","Ë"],
  ["Gamma","Γ"],
  ["Iacute","Í"],
  ["Icirc","Î"],
  ["Igrave","Ì"],
  ["Iota","Ι"],
  ["Iuml","Ï"],
  ["Kappa","Κ"],
  ["Lambda","Λ"],
  ["Mu","Μ"],
  ["Ntilde","Ñ"],
  ["Nu","Ν"],
  ["OElig","Œ"],
  ["Oacute","Ó"],
  ["Ocirc","Ô"],
  ["Ograve","Ò"],
  ["Omega","Ω"],
  ["Omicron","Ο"],
  ["Oslash","Ø"],
  ["Otilde","Õ"],
  ["Ouml","Ö"],
  ["Phi","Φ"],
  ["Pi","Π"],
  ["Prime","″"],
  ["Psi","Ψ"],
  ["Rho","Ρ"],
  ["Scaron","Š"],
  ["Sigma","Σ"],
  ["THORN","Þ"],
  ["Tau","Τ"],
  ["Theta","Θ"],
  ["Uacute","Ú"],
  ["Ucirc","Û"],
  ["Ugrave","Ù"],
  ["Upsilon","Υ"],
  ["Uuml","Ü"],
  ["Xi","Ξ"],
  ["Yacute","Ý"],
  ["Yuml","Ÿ"],
  ["Zeta","Ζ"],
  ["aacute","á"],
  ["acirc","â"],
  ["acute","´"],
  ["aelig","æ"],
  ["agrave","à"],
  ["alefsym","ℵ"],
  ["alpha","α"],
  ["amp","&"],
  ["and","∧"],
  ["ang","∠"],
  ["apos","'"],
  ["aring","å"],
  ["asymp","≈"],
  ["atilde","ã"],
  ["auml","ä"],
  ["bdquo","„"],
  ["beta","β"],
  ["brvbar","¦"],
  ["bull","•"],
  ["cap","∩"],
  ["ccedil","ç"],
  ["cedil","¸"],
  ["cent","¢"],
  ["chi","χ"],
  ["circ","ˆ"],
  ["clubs","♣"],
  ["cong","≅"],
  ["copy","©"],
  ["crarr","↵"],
  ["cup","∪"],
  ["curren","¤"],
  ["dArr","⇓"],
  ["dagger","†"],
  ["darr","↓"],
  ["deg","°"],
  ["delta","δ"],
  ["diams","♦"],
  ["divide","÷"],
  ["eacute","é"],
  ["ecirc","ê"],
  ["egrave","è"],
  ["empty","∅"],
  ["emsp"," "],
  ["ensp"," "],
  ["epsilon","ε"],
  ["equiv","≡"],
  ["eta","η"],
  ["eth","ð"],
  ["euml","ë"],
  ["euro","€"],
  ["exist","∃"],
  ["fnof","ƒ"],
  ["forall","∀"],
  ["frac12","½"],
  ["frac14","¼"],
  ["frac34","¾"],
  ["frasl","⁄"],
  ["gamma","γ"],
  ["ge","≥"],
  ["gt",">"],
  ["hArr","⇔"],
  ["harr","↔"],
  ["hearts","♥"],
  ["hellip","…"],
  ["iacute","í"],
  ["icirc","î"],
  ["iexcl","¡"],
  ["igrave","ì"],
  ["image","ℑ"],
  ["infin","∞"],
  ["int","∫"],
  ["iota","ι"],
  ["iquest","¿"],
  ["isin","∈"],
  ["iuml","ï"],
  ["kappa","κ"],
  ["lArr","⇐"],
  ["lambda","λ"],
  ["lang","〈"],
  ["laquo","«"],
  ["larr","←"],
  ["lceil","⌈"],
  ["ldquo","“"],
  ["le","≤"],
  ["lfloor","⌊"],
  ["lowast","∗"],
  ["loz","◊"],
  ["lrm","\xE2\x80\x8E"],
  ["lsaquo","‹"],
  ["lsquo","‘"],
  ["lt","<"],
  ["macr","¯"],
  ["mdash","—"],
  ["micro","µ"],
  ["middot","·"],
  ["minus","−"],
  ["mu","μ"],
  ["nabla","∇"],
  ["nbsp"," "],
  ["ndash","–"],
  ["ne","≠"],
  ["ni","∋"],
  ["not","¬"],
  ["notin","∉"],
  ["nsub","⊄"],
  ["ntilde","ñ"],
  ["nu","ν"],
  ["oacute","ó"],
  ["ocirc","ô"],
  ["oelig","œ"],
  ["ograve","ò"],
  ["oline","‾"],
  ["omega","ω"],
  ["omicron","ο"],
  ["oplus","⊕"],
  ["or","∨"],
  ["ordf","ª"],
  ["ordm","º"],
  ["oslash","ø"],
  ["otilde","õ"],
  ["otimes","⊗"],
  ["ouml","ö"],
  ["para","¶"],
  ["part","∂"],
  ["permil","‰"],
  ["perp","⊥"],
  ["phi","φ"],
  ["pi","π"],
  ["piv","ϖ"],
  ["plusmn","±"],
  ["pound","£"],
  ["prime","′"],
  ["prod","∏"],
  ["prop","∝"],
  ["psi","ψ"],
  ["quot","\""],
  ["rArr","⇒"],
  ["radic","√"],
  ["rang","〉"],
  ["raquo","»"],
  ["rarr","→"],
  ["rceil","⌉"],
  ["rdquo","”"],
  ["real","ℜ"],
  ["reg","®"],
  ["rfloor","⌋"],
  ["rho","ρ"],
  ["rlm","\xE2\x80\x8F"],
  ["rsaquo","›"],
  ["rsquo","’"],
  ["sbquo","‚"],
  ["scaron","š"],
  ["sdot","⋅"],
  ["sect","§"],
  ["shy","\xC2\xAD"],
  ["sigma","σ"],
  ["sigmaf","ς"],
  ["sim","∼"],
  ["spades","♠"],
  ["sub","⊂"],
  ["sube","⊆"],
  ["sum","∑"],
  ["sup","⊃"],
  ["sup1","¹"],
  ["sup2","²"],
  ["sup3","³"],
  ["supe","⊇"],
  ["szlig","ß"],
  ["tau","τ"],
  ["there4","∴"],
  ["theta","θ"],
  ["thetasym","ϑ"],
  ["thinsp"," "],
  ["thorn","þ"],
  ["tilde","˜"],
  ["times","×"],
  ["trade","™"],
  ["uArr","⇑"],
  ["uacute","ú"],
  ["uarr","↑"],
  ["ucirc","û"],
  ["ugrave","ù"],
  ["uml","¨"],
  ["upsih","ϒ"],
  ["upsilon","υ"],
  ["uuml","ü"],
  ["weierp","℘"],
  ["xi","ξ"],
  ["yacute","ý"],
  ["yen","¥"],
  ["yuml","ÿ"],
  ["zeta","ζ"],
  ["zwj","\xE2\x80\x8D"],
  ["zwnj","\xE2\x80\x8C"]
]);

const expr = /&(.+?);/g;
function repl(g, m) {
  var rv = entities.get(m);
  if (rv) {
    return rv;
  }
  if (m[0] == "#") {
    rv = parseInt(m.slice(1), 10);
    if (isFinite(rv)) {
      return String.fromCharCode(rv);
    }
  }
  return g;
}

function decodeEntities(s) {
  return s.replace(expr, repl);
}